/* Declarations for Checker and for the API.
   Copyright 1994, 1995 Tristan Gingold.
		  Written March 1994 by Tristan Gingold.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

The author may be reached by US/French mail:
		Tristan Gingold 
		8 rue Parmentier
		F-91120 PALAISEAU
		FRANCE
*/

#ifndef _CHECKER_H
#define _CHECKER_H

/* Note:  This file is used for the internals and for the API.  The API
  consists of functions that can be called by your program (to do garbage
  detection for example) and by your debugger (even if you don't need the
  declarartion.
  The API file, checker_api.h is build from checker.h: only lines between
  / * API * / and / * NOT-API * / (they must appears *alone* in a line,
  without blanks between the slahes and the asterisks.  */
  
#include <stddef.h>
#ifndef PTR
#define PTR void *
#endif
#ifdef __GNUC__
#define ATTRIBUTE(x) __attribute__(x)
#else
#define ATTRIBUTE (x)
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* These are the right in the bitmap.  A right is a 2 bits number.  */
/* At first the real right, which could be stored in a bitmap.  */
/* No rights:  can't neither be read nor written.  */
#define CHKR_UN	0

/* Read only: can only be read.  */
#define CHKR_RO	1

/* Write only: can only be written.  */
#define CHKR_WO	2

/* Read and write: can be read or written.  */
#define CHKR_RW 3

/* These are pseudo-right, used to test the bitmap.  See chkr_check_addr.  */
/* Test for write, but no evolution.  See chkr_check_addr.  */
#define CHKR_TW 6

/* Test for write and mark write-only.  */
#define CHKR_MW	10

/* Test for not a red zone.  */
#define CHKR_NOT_RZ 16

/* The real bits to consider.  */
#define CHKR_MASK 3

/* The exec right.  Never stored in a bitmap but in a descriptor.  */
#define CHKR_EXEC 4

void stubs_chkr_set_right (const PTR ptr, size_t len, unsigned char right);
int stubs_chkr_check_addr (const PTR ptr, size_t len, unsigned char right,
			   const char *mes);
int stubs_chkr_check_exec (const PTR ptr, const char *mes);
int stubs_chkr_check_str (const PTR ptr, unsigned char right, const char *mes);
int stubs_chkr_copy_bitmap (PTR dest, PTR src, size_t len, const char *mes);
void stubs_chkr_assert_not_red_zone (const PTR ptr, size_t len);

/* These functions are defined in error.c.  */
/* Fonction called ito abort.  */
void chkr_abort (void) ATTRIBUTE((noreturn));

/* Report an error.  Disp the header and the call chain.  */
int chkr_report (int i);

/* Just print the header message for an error.  */
int chkr_perror (int e);

/* Print the current call chain (aka history).  */
int chkr_disp_call_chain (void);

/* Return the number of reported messages.  */
int chkr_get_nbr_report (void);


/* Stupid function to call after each error reports.  To be breakpointed.  */
void __chkr_pause (void);

/* Cleaning functions after important syscalls.  They mainly update my_pid*
   and gather statistics.  */
void chkr_clean_after_fork (void);
void chkr_clean_before_exec (void);


/* Tell the manager that the program is using FD.  */
int fd_used_by_prog (int fd);

/* Tell the manager that the system has returned a new file descriptor FD.  */
int fd_returned_by_system (int fd);

/* Tell the manager that FD has been closed.  */
int fd_closed (int fd);

/* Tell the manager about dup() and dup2().  */
int fd_duped (int fdres, int fdd);


/* Defined in maccess.c, utils.c and garbage.c  */
struct mdesc;
int __chkr_read_bitmap (const PTR ptr, unsigned char *bm, size_t len);
int __chkr_write_bitmap (const PTR ptr, unsigned char *bm, size_t len);
void __chkr_disp_map (void);
void __chkr_disp_right (const PTR ptr, size_t len);
void __chkr_check_mdesc (struct mdesc *mdp);
void __chkr_check_intern (void);
void __chkr_disp_heaps (void);
void __chkr_disp_heapgraph (struct mdesc *mdp);
void __chkr_disp_statistics (struct mdesc *mdp);
void __chkr_dump_mheap (struct mdesc *mdp);
void __chkr_dump_heap (void);
void __chkr_dump_free (struct mdesc *mdp);
void __chkr_dump_block (PTR block);
void __chkr_disp_shlibs (void);
void __chkr_disp_siginfo (void);
void __chkr_disp_fd (void);
void __chkr_dump_symtab (void);
void __chkr_dump_sbusy (void);
void __chkr_garbage_detector (void);
void __chkr_disp_inuse (void);
void __chkr_evol_detector (void);
void __chkr_clear_leaks (void);
void __chkr_evol_inuse (void);
void __chkr_clear_inuse (void);
void __chkr_dump_suppress (void);


#if 0
extern PTR malloc (size_t size);
extern PTR realloc (PTR ptr, size_t size);
extern PTR calloc (size_t nmemb, size_t size);
extern void free (PTR ptr);
extern PTR memalign (size_t alignment, size_t size);
extern PTR valloc (size_t size);
extern PTR sbrk (int incr);
#endif

void chkr_clean_after_fork (void);
void chkr_clean_before_exec (void);

void chkr_printf (const char *message, ...) __attribute__ ((format (printf, 1, 2)));
void chkr_header (const char *message, ...) __attribute__ ((format (printf, 1, 2)));
extern int flag_weak_check_copy;

#ifdef __cplusplus
}
#endif


#endif /* checker.h  */
#define CHKR_PREFIX(x) "chkr."##x
